﻿using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;

namespace TestProject
{
    /// <summary>
    /// GIF 래퍼
    /// </summary>
    public class GIFWrapper : GIFPart.GIFPartList
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 헤더 - Header

        /// <summary>
        /// 헤더
        /// </summary>
        public GIFPartHeader Header
        {
            get
            {
                return this.partList == null || this.partList.Count == 0 ? null : this.partList[0] as GIFPartHeader;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - GIFWrapper()

        /// <summary>
        /// 생성자
        /// </summary>
        public GIFWrapper()
        {
            this.partList.Add(new GIFPartHeader());

            this.partList.Add(new GIFPartTerminator());
        }

        #endregion
        #region 생성자 - GIFWrapper(stream)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="stream">스트림</param>
        public GIFWrapper(Stream stream)
        {
            byte[] bufferByteArray;

            int length = 0;

            using(MemoryStream memoryStream = new MemoryStream())
            {
                bufferByteArray = new byte[65536];

                while(stream.Position < stream.Length)
                {
                    length = stream.Read(bufferByteArray, 0, bufferByteArray.Length);

                    memoryStream.Write(bufferByteArray, 0, length);
                }

                bufferByteArray = memoryStream.ToArray();
            }

            int index = 0;

            ParsePart(bufferByteArray, ref index);
        }

        #endregion
        #region 생성자 - GIFWrapper(filePath)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        public GIFWrapper(string filePath) : this(new FileStream(filePath, FileMode.Open, FileAccess.Read))
        {
        }

        #endregion
        #region 생성자 - GIFWrapper(dataByteArray)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="dataByteArray">데이터 바이트 배열</param>
        public GIFWrapper(byte[] dataByteArray)
        {
            int index = 0;

            ParsePart(dataByteArray, ref index);
        }

        #endregion
        #region 생성자 - GIFWrapper(partList)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="partList">GIF 파트 리스트</param>
        public GIFWrapper(IEnumerable<GIFPart> partList)
        {
            base.partList = partList.ToList();
        }

        #endregion
        #region 생성자 - GIFWrapper(imageEnumerable, animation, disposalMethod, delayTime, iterationCount)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="imageEnumerable">이미지 열거 가능 목록</param>
        /// <param name="animation">애니메이션 여부</param>
        /// <param name="disposalMethod">처분 방법</param>
        /// <param name="delayTime">지연 시간</param>
        /// <param name="iterationCount">반복 수</param>
        public GIFWrapper(IEnumerable<Image> imageEnumerable, bool animation, GIFPartMetaData.DisposalMethod disposalMethod, ushort delayTime, ushort iterationCount)
        {
            GIFPartHeader header = new GIFPartHeader();

            this.partList.Add(header);

            if(animation)
            {
                GIFPartMetaData.Loop loop = new GIFPartMetaData.Loop();

                loop.IterationCount = iterationCount;

                this.partList.Add(loop);
            }

            byte colorResolution = 0;

            foreach(Image image in imageEnumerable)
            {
                GIFPartMetaData.GraphicsControl control = new GIFPartMetaData.GraphicsControl();

                control.DelayTime      = delayTime;
                control.DisposalMethod = disposalMethod;

                this.partList.Add(control);

                GIFPartLWZImage lwzImage = GIFPartLWZImage.GetLWZImagePart(image, out colorResolution);

                lwzImage.Wrapper = this;

                this.partList.Add(lwzImage);

                if(lwzImage.Width > header.Width)
                {
                    header.Width = lwzImage.Width;
                }

                if(lwzImage.Height > header.Height)
                {
                    header.Height = lwzImage.Height;
                }

                if(colorResolution > header.ColorResolution)
                {
                    header.ColorResolution = colorResolution;
                }
            }

            this.partList.Add(new GIFPartTerminator());
        }

        #endregion
        #region 생성자 - GIFWrapper(imageEnumerable)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="imageEnumerable">이미지 열거 가능 목록</param>
        public GIFWrapper(IEnumerable<Image> imageEnumerable) : this(imageEnumerable, false, GIFPartMetaData.DisposalMethod.NotSpecified, 0, 0)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 추출하기 - Extract(lwzImage)

        /// <summary>
        /// 추출하기
        /// </summary>
        /// <param name="lwzImage">GIF 파트 LWZ 이미지</param>
        /// <returns>GIF 래퍼</returns>
        public GIFWrapper Extract(GIFPartLWZImage lwzImage)
        {
            GIFPartMetaData.GraphicsControl control = null;

            bool imageFound = false;

            foreach(GIFPart part in this.partList)
            {
                if(part is GIFPartMetaData.GraphicsControl)
                {
                    control = part as GIFPartMetaData.GraphicsControl;
                }

                if(part == lwzImage)
                {
                    imageFound = true;

                    break;
                }
            }

            if(!imageFound)
            {
                control = null;
            }

            List<GIFPart> temporaryPartList = new List<GIFPart>();

            temporaryPartList.Add(Header.CreateCopy());

            if(control != null)
            {
                temporaryPartList.Add(control.CreateCopy());
            }

            temporaryPartList.Add(lwzImage.CreateCopy());

            temporaryPartList.Add(new GIFPartTerminator());

            return new GIFWrapper(temporaryPartList);
        }

        #endregion
        #region 이미지 구하기 - ToImage()

        /// <summary>
        /// 이미지 구하기
        /// </summary>
        /// <returns>이미지</returns>
        public Image ToImage()
        {
            byte[] dataByteArray = GetDataByteArray();

            Image image = Image.FromStream(new MemoryStream(dataByteArray));

            return image;
        }

        #endregion
        #region 복사 생성하기 - CreateCopy()

        /// <summary>
        /// 복사 생성하기
        /// </summary>
        /// <returns>GIF 파트</returns>
        public override GIFPart CreateCopy()
        {
            return new GIFWrapper(this.partList.Select(p => p.CreateCopy()));
        }

        #endregion
        #region 지연 시간 설정하기 - SetDelayTime(delayTime)

        /// <summary>
        /// 지연 시간 설정하기
        /// </summary>
        /// <param name="delayTime">지연 시간</param>
        public void SetDelayTime(ushort delayTime)
        {
            foreach(GIFPart part in this.partList)
            {
                if(part is GIFPartMetaData.GraphicsControl)
                {
                    GIFPartMetaData.GraphicsControl control = part as GIFPartMetaData.GraphicsControl;

                    control.DelayTime = delayTime;
                }
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 파트 파싱하기 - ParsePart(dataByteArray, index)

        /// <summary>
        /// 파트 파싱하기
        /// </summary>
        /// <param name="dataByteArray">데이터 바이트 배열</param>
        /// <param name="index">인덱스</param>
        private void ParsePart(byte[] dataByteArray, ref int index)
        {
            bool terminated = false;

            while(index < dataByteArray.Length)
            {
                if(terminated)
                {
                    this.partList.Add(new GIFPartGarbageData(ref dataByteArray, ref index));
                }
                else if(index == 0)
                {
                    this.partList.Add(new GIFPartHeader(ref dataByteArray, ref index));
                }
                else if(dataByteArray[index] == 0x2C)
                {
                    this.partList.Add(new GIFPartLWZImage(this, ref dataByteArray, ref index));
                }
                else if(dataByteArray[index] == 0x21)
                {
                    this.partList.Add(GIFPartMetaData.Parse(ref dataByteArray, ref index));
                }
                else if(dataByteArray[index] == 0x3B)
                {
                    this.partList.Add(new GIFPartTerminator(ref dataByteArray, ref index));

                    terminated = true;
                }
                else
                {
                    throw new GIFParserException(index, string.Format("Failed to parse GIF: 0x{0:X2} is not a valid part identifier!", dataByteArray[index]));
                }
            }
        }

        #endregion
    }
}