﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// GIF 색상 테이블 컨트롤
    /// </summary>
    public partial class GIFColorTableControl : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 변경시 이벤트 - ValueChanged

        /// <summary>
        /// 값 변경시 이벤트
        /// </summary>
        public event EventHandler ValueChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 컬럼 수
        /// </summary>
        private const int COLUMN_COUNT = 16;

        /// <summary>
        /// 타일 크기
        /// </summary>
        private const int TILE_SIZE = 32;

        /// <summary>
        /// 색상 리스트
        /// </summary>
        private IList<Color> colorList = new List<Color>();

        /// <summary>
        /// 행 수
        /// </summary>
        private int rowCount = 0;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public IList<Color> Value
        {
            get
            {
                return this.colorList;
            }
            set
            {
                this.colorList = value;

                UpdateColorPictureBox();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - GIFColorTableControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public GIFColorTableControl()
        {
            InitializeComponent();

            this.colorPictureBox.Resize     += colorPictureBox_Resize;
            this.colorPictureBox.MouseClick += colorPictureBox_MouseClick;

            UpdateColorPictureBox();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 색상 픽처 박스 크기 변경시 처리하기 - colorPictureBox_Resize(sender, e)

        /// <summary>
        /// 색상 픽처 박스 크기 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void colorPictureBox_Resize(object sender, EventArgs e)
        {
            this.colorPictureBox.Height = this.colorPictureBox.Width * this.rowCount / COLUMN_COUNT;
        }

        #endregion
        #region 색상 픽처 박스 마우스 클릭시 처리하기 - colorPictureBox_MouseClick(sender, e)

        /// <summary>
        /// 색상 픽처 박스 마우스 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void colorPictureBox_MouseClick(object sender, MouseEventArgs e)
        {
            if(!Enabled)
            {
                return;
            }

            if(e.Button == MouseButtons.Left)
            {
                int x = COLUMN_COUNT * e.X / colorPictureBox.Width;
                int y = e.Y * this.rowCount / colorPictureBox.Height;

                int index = COLUMN_COUNT * y + x;

                if(index < this.colorList.Count)
                {
                    ProcessClickColor(index);
                }
            }
        }

        #endregion


        //////////////////////////////////////////////////////////////////////////////// Function

        #region 값 변경시 이벤트 발생시키기 - FireValueChangedEvent()

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        private void FireValueChangedEvent()
        {
            if(ValueChanged != null)
            {
                try
                {
                    ValueChanged(this, EventArgs.Empty);
                }
                catch
                {
                }
            }
        }

        #endregion
        #region 색상 픽처 박스 업데이트하기 - UpdateColorPictureBox()

        /// <summary>
        /// 색상 픽처 박스 업데이트하기
        /// </summary>
        private void UpdateColorPictureBox()
        {
            this.rowCount = Math.Max(1, ((this.colorList.Count - 1) / COLUMN_COUNT) + 1);

            Bitmap bitmap = new Bitmap(COLUMN_COUNT * TILE_SIZE, this.rowCount * TILE_SIZE);

            using(Graphics graphics = Graphics.FromImage(bitmap))
            {
                int x = 0;
                int y = 0;

                foreach(Color color in this.colorList)
                {
                    graphics.DrawRectangle(Pens.Black, x * TILE_SIZE, y * TILE_SIZE, TILE_SIZE - 1, TILE_SIZE - 1);

                    using(SolidBrush brush = new SolidBrush(color))
                    {
                        graphics.FillRectangle(brush, x * TILE_SIZE + 1, y * TILE_SIZE + 1, TILE_SIZE - 3, TILE_SIZE - 3);
                    }

                    x++;

                    if(x == COLUMN_COUNT)
                    {
                        x = 0;

                        y++;
                    }
                }
            }

            if(this.colorPictureBox.Image != null)
            {
                this.colorPictureBox.Image.Dispose();
            }

            this.colorPictureBox.Image  = bitmap;
            this.colorPictureBox.Height = this.colorPictureBox.Width * this.rowCount / COLUMN_COUNT;

            GC.Collect();
        }

        #endregion
        #region 색상 클릭시 처리하기 - ProcessClickColor(index)

        /// <summary>
        /// 색상 클릭시 처리하기
        /// </summary>
        /// <param name="index">인덱스</param>
        private void ProcessClickColor(int index)
        {
            Color color = this.colorList[index];

            ColorDialog colorDialog = new ColorDialog();

            colorDialog.Color = color;

            if(colorDialog.ShowDialog() == DialogResult.OK)
            {
                this.colorList[index] = colorDialog.Color;

                UpdateColorPictureBox();

                FireValueChangedEvent();
            }
        }

        #endregion
    }
}