﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// GIF 파트 LWZ 이미지 컨트롤
    /// </summary>
    public partial class GIFPartLWZImageControl : UserControl, IGIFControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 변경시 이벤트 - ValueChanged

        /// <summary>
        /// 값 변경시 이벤트
        /// </summary>
        public event EventHandler ValueChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 파트
        /// </summary>
        private GIFPartLWZImage part;

        /// <summary>
        /// 체크 여부
        /// </summary>
        private bool isChecked = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 파트 - Part

        /// <summary>
        /// 파트
        /// </summary>
        public GIFPartLWZImage Part
        {
            get
            {
                return this.part;
            }
            set
            {
                this.isChecked = false;

                this.part = value;

                if(value == null)
                {
                    this.leftNumericUpDown.Value            = 0;
                    this.topNumericUpDown.Value             = 0;
                    this.widthNumericUpDown.Value           = 0;
                    this.heightNumericUpDown.Value          = 0;
                    this.minimumCodeSizeNumericUpDown.Value = 0;
                    this.colorTableSizeNumericUpDown.Value  = 0;
                    this.interlaceCheckBox.Checked          = false;
                    this.sortColorTableCheckBox.Checked     = false;
                    this.useLocalColorTableCheckBox.Checked = false;

                    this.colorTable.Value = new List<Color>();
                }
                else
                {
                    this.leftNumericUpDown.Value            = value.Left;
                    this.topNumericUpDown.Value             = value.Top;
                    this.widthNumericUpDown.Value           = value.Width;
                    this.heightNumericUpDown.Value          = value.Height;
                    this.minimumCodeSizeNumericUpDown.Value = value.MinimumCodeSize;
                    this.colorTableSizeNumericUpDown.Value  = value.UseLocalColorTable ? value.LocalColorTableSize : 0;
                    this.interlaceCheckBox.Checked          = value.Interlaced;
                    this.sortColorTableCheckBox.Checked     = value.TableSorted;
                    this.useLocalColorTableCheckBox.Checked = value.UseLocalColorTable;

                    this.colorTable.Value = value.LocalColorTable ?? new List<Color>();
                }

                UpdatePreviewPictureBox();

                this.isChecked = true;
            }
        }

        #endregion
        #region GIF 파트 - GIFPart

        /// <summary>
        /// GIF 파트
        /// </summary>
        public GIFPart GIFPart
        {
            get
            {
                return this.part;
            }
            set
            {
                Part = value as GIFPartLWZImage;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - GIFPartLWZImageControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public GIFPartLWZImageControl()
        {
            InitializeComponent();

            this.leftNumericUpDown.ValueChanged            += (sender, e) => { FireValueChangedEvent(sender as NumericUpDown, v => this.part.Left = v                           ); };
            this.topNumericUpDown.ValueChanged             += (sender, e) => { FireValueChangedEvent(sender as NumericUpDown, v => this.part.Top = v                            ); };
            this.widthNumericUpDown.ValueChanged           += (sender, e) => { FireValueChangedEvent(sender as NumericUpDown, v => this.part.Width = v                          ); };
            this.heightNumericUpDown.ValueChanged          += (sender, e) => { FireValueChangedEvent(sender as NumericUpDown, v => this.part.Height = v                         ); };
            this.minimumCodeSizeNumericUpDown.ValueChanged += (sender, e) => { FireValueChangedEvent(sender as NumericUpDown, v => this.part.MinimumCodeSize = Convert.ToByte(v)); };
            this.interlaceCheckBox.CheckedChanged          += (sender, e) => { FireValueChangedEvent(sender as CheckBox     , v => this.part.Interlaced = v                     ); };
            this.sortColorTableCheckBox.CheckedChanged     += (sender, e) => { FireValueChangedEvent(sender as CheckBox     , v => this.part.TableSorted = v                    ); };
            this.colorTable.ValueChanged                   += (sender, e) => {
            
                if(this.part != null && this.isChecked)
                {
                    FireValueChangedEvent();
                }
            };

            this.deleteTableButton.Click    += deleteTableButton_Click;
            this.importButton.Click         += importButton_Click;
            this.transformColorButton.Click += transformColorButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 테이블 삭제 버튼 클릭시 처리하기 - deleteTableButton_Click(sender, e)

        /// <summary>
        /// 테이블 삭제 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void deleteTableButton_Click(object sender, EventArgs e)
        {
            if(this.part == null)
            {
                return;
            }

            try
            {
                if(this.part.UseLocalColorTable)
                {
                    if(this.part.Wrapper == null)
                    {
                        throw new Exception("No image connection set!");
                    }
                    else if(!this.part.Wrapper.Header.UseGlobalColorTable)
                    {
                        throw new Exception("There is no global color table to use!");
                    }
                    else if(this.part.LocalColorTable.Count != this.part.Wrapper.Header.GlobalColorTable.Count)
                    {
                        throw new Exception("The color count does not match!");
                    }
                    else
                    {
                        this.part.LocalColorTable = null;
                    }
                }

                Part = this.part;

                FireValueChangedEvent();
            }
            catch(Exception exception)
            {
                MessageBox.Show
                (
                    exception.Message,
                    "ERROR",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error
                );
            }
        }

        #endregion
        #region 가져오기 버튼 클릭시 처리하기 - importButton_Click(sender, e)

        /// <summary>
        /// 가져오기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void importButton_Click(object sender, EventArgs e)
        {
            if(this.part == null)
            {
                return;
            }

            if(this.part.Wrapper == null)
            {
                MessageBox.Show
                (
                    "이미지 컬렉션 세트가 없습니다!",
                    "ERROR",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error
                );

                return;
            }

            try
            {
                if(!this.part.Wrapper.Header.UseGlobalColorTable)
                {
                    throw new Exception("There is no global color table!");
                }
                else if(this.part.UseLocalColorTable)
                {
                    if(this.part.LocalColorTable.Count != this.part.Wrapper.Header.GlobalColorTable.Count)
                    {
                        throw new Exception("The color count does not match!");
                    }
                    else
                    {
                        this.part.LocalColorTable = this.part.Wrapper.Header.GlobalColorTable;
                    }
                }
                else
                {
                    this.part.LocalColorTable = this.part.Wrapper.Header.GlobalColorTable;
                }

                Part = this.part;

                FireValueChangedEvent();
            }
            catch(Exception exception)
            {
                MessageBox.Show
                (
                    exception.Message,
                    "ERROR",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error
                );
            }
        }

        #endregion
        #region 색상 변환 버튼 클릭시 처리하기 - transformColorButton_Click(sender, e)

        /// <summary>
        /// 색상 변환 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void transformColorButton_Click(object sender, EventArgs e)
        {
            if(colorTable.Value == null || colorTable.Value.Count == 0)
            {
                return;
            }

            GIFColorTableTransformForm form = new GIFColorTableTransformForm(colorTable.Value);

            if(form.ShowDialog() == DialogResult.OK)
            {
                for(int i = 0; i < form.Value.Count; i++)
                {
                    colorTable.Value[i] = form.Value[i];
                }

                colorTable.Value = colorTable.Value;

                FireValueChangedEvent();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 값 변경시 이벤트 발생시키기 - FireValueChangedEvent()

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        private void FireValueChangedEvent()
        {
            if(ValueChanged != null)
            {
                try
                {
                    ValueChanged(this, EventArgs.Empty);

                    UpdatePreviewPictureBox();
                }
                catch
                {
                }
            }
        }

        #endregion
        #region 값 변경시 이벤트 발생시키기 - FireValueChangedEvent(numericUpDown, action)

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="numericUpDown">숫자 UP/DOWN</param>
        /// <param name="action">액션</param>
        private void FireValueChangedEvent(NumericUpDown numericUpDown, Action<ushort> action)
        {
            if(this.part == null)
            {
                return;
            }

            if(!this.isChecked)
            {
                return;
            }

            try
            {
                ushort value = Convert.ToUInt16(numericUpDown.Value);

                action(value);

                FireValueChangedEvent();
            }
            catch
            {
            }
        }

        #endregion
        #region 값 변경시 이벤트 발생시키기 - FireValueChangedEvent(checkBox, action)

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="checkBox">체크 박스</param>
        /// <param name="action">액션</param>
        private void FireValueChangedEvent(CheckBox checkBox, Action<bool> action)
        {
            if(this.part == null)
            {
                return;
            }

            if(!this.isChecked)
            {
                return;
            }

            try
            {
                bool value = checkBox.Checked;

                action(value);

                FireValueChangedEvent();
            }
            catch
            {
            }
        }

        #endregion

        #region 미리보기 픽처 박스 업데이트하기 - UpdatePreviewPictureBox()

        /// <summary>
        /// 미리보기 픽처 박스 업데이트하기
        /// </summary>
        private void UpdatePreviewPictureBox()
        {
            if(InvokeRequired)
            {
                Invoke(new Action(UpdatePreviewPictureBox));

                return;
            }

            Image image = null;

            if(this.part != null)
            {
                try
                {
                    image = this.part.ToImage();
                }
                catch(Exception excaption)
                {
                    image = new Bitmap(256, 256);

                    using(Graphics graphics = Graphics.FromImage(image))
                    {
                        using(Pen pen = new Pen(Color.Red, 4f))
                        {
                            using(Font font = new Font(FontFamily.GenericSansSerif, 8f, FontStyle.Regular))
                            {
                                graphics.Clear(Color.Transparent);

                                graphics.DrawRectangle(pen, 0, 0, 256, 256);

                                graphics.DrawLine(pen, 0, 0, 256, 256);

                                graphics.DrawLine(pen, 256, 0, 0, 256);

                                graphics.DrawString(excaption.Message, font, Brushes.Black, new RectangleF(8, 8, 248, 248));
                            }
                        }
                    }
                }
            }

            if(this.previewPictureBox.Image != null)
            {
                this.previewPictureBox.Image.Dispose();
            }

            this.previewPictureBox.Image = image;
        }

        #endregion
    }
}